/**********************************************************************
 **                                                                  **
 **                            callbacks.cc                          **
 **                                                                  **
 **                                                                  **
 **  This files contains all the callback functions of eBug.         **
 **                                                                  **
 **  Written by:  Yoichiro Endo (Mobile Robot Lab, Georgia Tech)     **
 **                                                                  **
 **  Copyright 2000 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: callbacks.cc,v 1.6 2000/10/25 04:44:32 endo Exp $ */

#include "ebug.h"

//-----------------------------------------------------------------------------
// This functions gets called when button is clicked in drawing area or key
// pressed.
//-----------------------------------------------------------------------------
void kb_input_cb(Widget w, XtPointer data, XmDrawingAreaCallbackStruct *callback_data)
{
  ScreenUtility   screen_utility;
  LightFunction   light_function;

  char            buffer[16];
  KeySym          ks;
  XComposeStatus  cs;
  int             len;

  if (event.type == KeyPress)
  {
    len = XLookupString((XKeyEvent *)&event, buffer, 16, &ks, &cs);

    switch(buffer[0]) {    
    case 'q':
    case 'x':
      XFreeGC(display, gc);
      XCloseDisplay(display);
      exit(1);
    case 's':
      light_function.save();
      break;
    default:
      break;
    }
  }
  else if (event.type == ButtonPress)
  {
    point pt;
    pt.x = event.xbutton.x;
    pt.y = height - event.xbutton.y;

    int id;
    switch (((XButtonEvent *)&event)->button) {
    case 1:
      if (light_function.inLight(pt, &id))
	light_function.changeType(id);
      else
	light_function.add(pt);
      break;
    case 2:
      if (light_function.inLight(pt, &id))
	light_function.changeStatus(id, TOGGLE);
      break;
    case 3:
      if (light_function.inLight(pt, &id))
	light_function.changeSize(id);
      break;
    default:
      break;
    }
  }
  screen_utility.getXEvent();
  return;
}

//-----------------------------------------------------------------------------
// This function gets called when the user modifies the value of the Hypothesis
// value in the window.
//-----------------------------------------------------------------------------
void updateHypothesisValue_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
  int type = (int)client_data;
  int val;

  XmScaleGetValue(hypothesis_scale[type], &val);

  t_node *pr = robotList;
  while (pr->next != NULL)
  {
    t_robot *robot = (t_robot *)pr->data;
    robot->hypothesis[type] = (double)val;

    pr = pr->next;
  }
}

//-----------------------------------------------------------------------------
// This function gets called when the user modifies the value of the 
// Orientation Need value in the window.
//-----------------------------------------------------------------------------
void updateOrientationNeedValue_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
  ScreenUtility screen_utility;

  screen_utility.clearWindow();

  int type = (int)client_data;
  int val;

  XmScaleGetValue(orientation_need_scale[type], &val);

  t_node *pr = robotList;
  while (pr->next != NULL)
  {
    t_robot *robot = (t_robot *)pr->data;
    robot->orientation_need[type] = (double)val;

    pr = pr->next;
  }
}

//-----------------------------------------------------------------------------
// This function gets called when the user pushes the "Load Light" button.
//-----------------------------------------------------------------------------
void pushLoadLightsButton_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
  ListUtility    list_utility;
  ScreenUtility  screen_utility;
  LightFunction  light_function;

  char fname[100];
  printf("File name: ");
  scanf("%s",fname);

  screen_utility.drawLights(ERASE);
  list_utility.clearList(&lightList);
  light_function.load(fname);
  screen_utility.clearWindow();
  screen_utility.drawLights(DRAW);
}

//-----------------------------------------------------------------------------
// This function gets called when the user pushes the "Run/Pause" button.
//-----------------------------------------------------------------------------
void pushRunPauseButton_cb(Widget w, XtPointer client_data, XtPointer call_data)
{

  ScreenUtility   screen_utility;
  HardwareCommand hardware_command;

  if (go)
  {
    go = FALSE;

    if (run_type == HSERVER)
      hardware_command.stopRobot();
  }
  else
    go = TRUE;

  XtAppAddWorkProc(app, (XtWorkProc)ebugMainLoop, NULL);
  screen_utility.updateRunPauseButton();
}

//-----------------------------------------------------------------------------
// This function gets called when the user pushes the "Reset" button.
//-----------------------------------------------------------------------------
void pushResetButton_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
  TimeUtility     time_utility;
  ScreenUtility   screen_utility;
  HardwareCommand hardware_command;

  if (run_type == HSERVER)
    hardware_command.stopRobot();

  go = FALSE;
  screen_utility.updateRunPauseButton();

  time_utility.nap(1.0);
  initializeExperiment(PARTIAL);

}

//-----------------------------------------------------------------------------
// This function gets called when the user pushes the "Quit" button.
//-----------------------------------------------------------------------------
void pushQuitButton_cb(Widget w, XtPointer client_data, XtPointer call_data)
{
  HardwareCommand hardware_command;
  LogUtility      log_utility;
  

  if (run_type == HSERVER)
    hardware_command.disconnectRobot();

  if (log_data)
    log_utility.closeLogFile();

  XFreeGC(display, gc);
  XCloseDisplay(display);
  exit(1);
}

/**********************************************************************
 * $Log: callbacks.cc,v $
 * Revision 1.6  2000/10/25 04:44:32  endo
 * Code cleaned up for the distribution.
 *
 * Revision 1.5  2000/08/19 20:43:39  endo
 * Slight changes due to the hardware interface.
 *
 * Revision 1.4  2000/08/07 21:00:28  endo
 * Reset feature added.
 *
 * Revision 1.3  2000/08/07 01:45:45  endo
 * Orientation Need Window created.
 *
 * Revision 1.2  2000/08/06 07:28:26  endo
 * Two light types implemented successfully.
 *
 * Revision 1.1  2000/08/05 19:40:23  endo
 * Initial revision
 *
 *
 **********************************************************************/
