/**********************************************************************
 **                                                                  **
 **                           hardware.cc                            **
 **                                                                  **
 **                                                                  **
 **  This file contains the functions for HardwareCommand class,     **
 **  which deals the commands to be sent to HServer.                 **
 **                                                                  **
 **  Written by:  Yoichiro Endo (Mobile Robot Lab, Georgia Tech)     **
 **                                                                  **
 **  Copyright 2000 Georgia Tech Research Corporation                **
 **  Atlanta, Georgia  30332-0415                                    **
 **  ALL RIGHTS RESERVED, See file COPYRIGHT for details.            **
 **                                                                  **
 **********************************************************************/

/* $Id: hardware.cc,v 1.5 2000/10/25 04:44:32 endo Exp $ */

extern "C" {
#include "ipt/ipt.h"
#include "ipt/message.h"
#include "ipt/connection.h"
}
#include "ebug.h"
#include "hclient.h"
#include "hserver/hserver.h"

IPCommunicator* communicator;
IPConnection* console;

char hserver_name[256]   = "fred";
float danger_range       = 0.5;
float drive_wait_angle   = 180;
int use_reverse          = FALSE;
bool use_sonar           = FALSE;
bool use_laser           = FALSE;
bool use_cognachrome     = TRUE;

float MAX_SPEED          = 0.25;
double MIN_DIST          = 250;
double lightlife         = 1.75;

//-----------------------------------------------------------------------------
// HardwareCommand is a class which is a collection of functions that deal with
// the commands to be sent to HServer when the run_type is HSERVER.
//-----------------------------------------------------------------------------
HardwareCommand::HardwareCommand()
{
}

//-----------------------------------------------------------------------------
// This function initializes the communication setup with HServer.
//-----------------------------------------------------------------------------
void HardwareCommand::initialize(int init_type)
{
  Calculate calculate;

  if (init_type == ALL)
  {
    char ipt_module_name[256];
    char hostname[100];
    char *ipt_home = NULL;

    sprintf(ipt_module_name, "ebug_%d", getpid());

    // Find the IPTServer on the net.
    if( ipt_home == NULL ) ipt_home = getenv("IPTHOST");
    if( ipt_home == NULL ) ipt_home = getenv("TCXHOST");
    if( ipt_home == NULL ) ipt_home = getenv("HOST");
  
    if ( ipt_home == NULL ) 
    {
      if (gethostname(hostname, 100) != 0) 
      {
	fprintf(stderr,"Error: Unable to determine host for IPTServer!\n");
	exit(1);
      }
      ipt_home = hostname;
    }

    // Connect to the IPTServer.
    fprintf(stderr,"Initialize IPT (%s, %s)\n",ipt_module_name, ipt_home );
    communicator = IPCommunicator::Instance(ipt_module_name, ipt_home);

    // Open the communication with HServer using the HClient command.
    if (hclient_open_robot(hserver_name, FALSE))
    {
      fprintf(stderr, "Error: initializing robot hardware");
      exit(1);
    }

    // Stop the sonar sensor reading.
    if (hclient_range_stop())
    {
      fprintf(stderr, "Error: stopping ultrasonics");
      exit(1);
    }
  }

  // Get the robot list.
  t_node *pr = robotList;

  while (pr->next != NULL)
  {
    // Get an individual robot start position.
    t_robot *robot = (t_robot *)pr->data;
    double x = robot->x;
    double y = robot->y;
    double theta = robot->theta;
    calculate.convertCoordinate(&x, &y, &theta, EBUG, HSERVER);

    // Set the robot position.
    if (hclient_setxyt(x, y, theta))
    {
      fprintf(stderr, "Error calling hclient_setxy(%f,%f,%f)\n", robot->x, robot->y, robot->theta);
      exit (1);
    }
    pr = pr->next;
  }

  // Start sonar sensor reading.
  hclient_update_sensors();

  if( debug ) fprintf(stderr,"Robot initialized\n");
}  

//-----------------------------------------------------------------------------
// This function gets the information about the robot position from HServer.
//-----------------------------------------------------------------------------
void HardwareCommand::getCurrentRobotInfo(void)
{
  Calculate calculate;
  double    x, y, theta;

  t_node *pr = robotList;
  while (pr->next != NULL)
  {
    t_robot *robot = (t_robot *)pr->data;

    hclient_update_sensors();
    if (hclient_getxyt(&x, &y, &theta)) fprintf(stderr,"Error in hclient_getxyt\n");
    calculate.convertCoordinate(&x, &y, &theta, HSERVER, EBUG);

    robot->x = x;
    robot->y = y;
    robot->theta = theta;

    pr = pr->next;
  }
}

//-----------------------------------------------------------------------------
// This function gets the information about the stimuli from HServer.
//-----------------------------------------------------------------------------
void HardwareCommand::getCurrentLightInfo(void)
{
  Calculate        calculate;
  ScreenUtility    screen_utility;
  LightFunction    light_function;
  sensor_reading_t *reading;

  if (!use_cognachrome) return;

  if (lightList->data)
  {
    light_function.updateLightAge();
    light_function.deleteOldLights(lightlife);
  }

  point pt;
  int mask = SENSOR_COGNACHROME_MASK;
  int num_readings = hclient_get_obstacles(mask, 0.15, &reading);

  t_node *pr = robotList;

  while (pr->next != NULL)
  {
    t_robot *robot = (t_robot *)pr->data;

    double dist, angle, x, y;
    double closest_dist = BIGNUMBER;
    t_light *closest_light;
    int id;

    for(int i=0;i<num_readings;i++)
    {
      dist = reading[i].reading/SIM_TO_REAL;
      angle = reading[i].angle - robot->theta;

      x = cos(angle*(M_PI/180.0))*dist + robot->x;
      y = sin(angle*(M_PI/180.0))*dist + robot->y;
      calculate.rotatePoint(robot->theta, &x, &y, robot->x, robot->y);

      pt.x = (int)x;
      pt.y = (int)y;


      switch(reading[i].color){
      case 0:
	if (light_function.inLight(pt, &id))
	  if (light_function.lightType(id) == TYPE_A) break;
	light_function.add(pt);
	break;
      case 1:
	if (light_function.inLight(pt, &id))
	  if (light_function.lightType(id) == TYPE_B) break;
	light_function.add(pt);
	light_function.changeType(lightId-1);
	break;
      case 2:
	/*
	if ((!go) || (dist > MIN_DIST)) break;
	if (lightList->data != NULL)
	{
	  closest_light= light_function.closestLight(robot, &closest_dist);
	  robot->hypothesis[closest_light->type] += 10;
	}
	else if (robot->last_seen_light_type != NO_TYPE)
	{
	  robot->hypothesis[robot->last_seen_light_type] += 10;
	  robot->last_seen_light_type = NO_TYPE;
	}
	go = FALSE;
	screen_utility.updateRunPauseButton();
	*/
	break;
      case 3:
	break;
      }
    }
    if ((go) && (lightList->data != NULL))
    {
      closest_light = light_function.closestLight(robot, &closest_dist);
      robot->last_seen_light_type = closest_light->type;
    }
    pr = pr->next;
  }
}

//-----------------------------------------------------------------------------
// This function sends the command to HServer, so that the robot be driven.
//-----------------------------------------------------------------------------
void HardwareCommand::driveRobot(void)
{
  Calculate calculate;

  t_node *pr = robotList;
  while (pr->next != NULL)
  {
    t_robot *robot = (t_robot *)pr->data;

    double speed = robot->progression;
    double angvel = robot->angvel;
    calculate.convertVelocity(&speed, LINEAR, EBUG, HSERVER);
    calculate.convertVelocity(&angvel, ANGULAR, EBUG, HSERVER);

    if (speed >= MAX_SPEED) speed = MAX_SPEED;
    hclient_drive((float)speed);
    hclient_steer((float)angvel);
    //hclient_steer_toward_and_drive((float)speed, 0, drive_wait_angle, use_reverse);

    pr = pr->next;
  }
}

//-----------------------------------------------------------------------------
// This function sends the command to HServer to stop the robot.
//-----------------------------------------------------------------------------
void HardwareCommand::stopRobot(void)
{
  t_node *pr = robotList;
  while (pr->next != NULL)
  {
    hclient_stop();
    /*
    hclient_steer(0.0);
    hclient_drive(0.0);
    */
    pr = pr->next;
  }
}

//-----------------------------------------------------------------------------
// This function sends the command to HServer to disconnect the robot.
//-----------------------------------------------------------------------------
void HardwareCommand::disconnectRobot(void)
{
  t_node *pr = robotList;
  while (pr->next != NULL)
  {
    t_robot *robot = (t_robot *)pr->data;

    hclient_steer(0.0);
    hclient_drive(0.0);
    hclient_range_stop();
    hclient_close_robot();

    pr = pr->next;
  }
}

/**********************************************************************
 * $Log: hardware.cc,v $
 * Revision 1.5  2000/10/25 04:44:32  endo
 * Code cleaned up for the distribution.
 *
 * Revision 1.4  2000/08/19 20:42:21  endo
 * Interface with HServer improved.
 *
 * Revision 1.3  2000/08/13 09:00:38  endo
 * getCurrentLightInfo added.
 *
 * Revision 1.2  2000/08/10 06:14:46  endo
 * driveRobot, stopRobot etc.
 *
 * Revision 1.1  2000/08/08 09:09:36  endo
 * Initial revision
 *
 **********************************************************************/
